#!/usr/bin/perl -w
# $Id$

use strict;

use POE;

use POE::Wheel::Run;
use Data::Denter;
use Test::More ( tests => 14 );
use FindBin;

sub DEBUG () { 1 }

my $object = bless {}, 'Test';

diag( "This test takes about 20 seconds" );



POE::Session->create(
    inline_states => {
        node_stdout   => sub { $object->node_stdout( @_[ARG0..$#_] ) },
        node_stderr   => sub { $object->node_stderr( @_[ARG0..$#_] ) },
        node2_stdout   => sub { $object->node2_stdout( @_[ARG0..$#_] ) },
        node2_stderr   => sub { $object->node2_stderr( @_[ARG0..$#_] ) },
        server_stdout => sub { $object->server_stdout( @_[ARG0..$#_] ) },
        server_stderr => sub { $object->server_stderr( @_[ARG0..$#_] ) },
        server_close  => sub { $object->server_close( @_[ARG0..$#_] ) },
        client_stdout => sub { $object->client_stdout( @_[ARG0..$#_] ) },
        client_stderr => sub { $object->client_stderr( @_[ARG0..$#_] ) },
        client_close  => sub { $object->client_close( @_[ARG0..$#_] ) },
        
        error         => sub { $object->error( @_[ARG0..$#_] ) },
        shutdown      => sub { $object->shutdown( @_[ARG0..$#_] ) },
        _start        => sub { $object->start( @_[ARG0..$#_] ) },
        start2        => sub { $object->start2( @_[ARG0..$#_] ) },
        start3        => sub { $object->start3( @_[ARG0..$#_] ) },
    }
);


$poe_kernel->run();

ok(1, "Everything exited");

foreach my $n ( 'Done', 'Running server', 'Sending time', 'Creating sessions', 
            'Running client' ) {
    is( $object->{$n}, 1, "Got $n once" );
}

ok( ($object->{client_connect}{'IKC-node'} and
     $object->{client_connect}{'LISTEN-0.0.0.0:12345'} ), 
                "Client saw proxy node" );

ok( $object->{client_connect}{Pulse}, "Client saw server" );

ok( $object->{client_connect}{HONKER}, "Client saw server register later" );

is( keys(%{$object->{client_pulse}}), 1, "Only one process saw a pulse" );
my $client = (keys %{$object->{client_pulse}})[0];

is( $object->{client_pulse}{$client}, 5, "Client saw 5 pulses" );

pass( "If we got this far, it means that subscribe worked through the proxy" );


ok( $object->{node2_PID}, "Second proxy node launched" ); 
is( $object->{node2_in_use}, 1, "Address was in use for second proxy node" );

# warn Denter $object;

##########################################################################
package Test;

use strict;
use POE;

use Config;

my $perl;

BEGIN { 
    *DEBUG = \&main::DEBUG;
    $perl = $Config{perl5} || $Config{perlpath};
}

sub start
{
    my( $self ) = @_;

    DEBUG and warn "Launching ikc-node\n";
    $self->{node_wheel} = POE::Wheel::Run->new( 
        Program => "$perl -Iblib/lib $FindBin::Bin/../ikc-node --verbose :12345",
        ErrorEvent => 'error',
        StdoutEvent => 'node_stdout',
        StderrEvent => 'node_stderr',
    );
    $poe_kernel->delay( 'start2', 2 );
}

sub start2
{
    my( $self ) = @_;

    DEBUG and warn "Launching node-client\n";
    $self->{client_wheel} = POE::Wheel::Run->new( 
        Program => "$perl -Iblib/lib $FindBin::Bin/node-client",
        ErrorEvent => 'error',
        CloseEvent => 'client_close',
        StdoutEvent => 'client_stdout',
        StderrEvent => 'client_stderr',
    );

    $poe_kernel->delay( 'start3', 2 );
}

sub start3
{
    my( $self ) = @_;

    DEBUG and warn "Launching node-server\n";
    $self->{server_wheel} = POE::Wheel::Run->new( 
        Program => "$perl -Iblib/lib $FindBin::Bin/node-server",
        ErrorEvent => 'error',
        CloseEvent => 'server_close',
        StdoutEvent => 'server_stdout',
        StderrEvent => 'server_stderr',
    );

    DEBUG and warn "Launching another ikc-node\n";
    $self->{node_wheel} = POE::Wheel::Run->new( 
        Program => "$perl -Iblib/lib $FindBin::Bin/../ikc-node --verbose :12345",
        ErrorEvent => 'error',
        StdoutEvent => 'node2_stdout',
        StderrEvent => 'node2_stderr',
    );

}

sub error
{
    my( $self, $syscall, $errno, $errstr, $id, $handle ) = @_;

    return if $errno == 0;

    warn "Error in wheel $id during $syscall: $errno ($errstr) handle=$handle\n";
}

######################################################
sub node_stdout
{
    my( $self, $input, $wid ) = @_;

    DEBUG and warn "NODE: $input\n";
    if( $input =~ /PID=(\d+)/ ) {
        $self->{node_PID} = $1;
        END { kill 15, $self->{node_PID} if $self->{node_PID} };
    }
    else {
        push @{ $self->{node_output} }, $input;
    }
}

sub node_stderr
{
    my( $self, $input, $wid ) = @_;

    DEBUG and warn "NODE err: $input\n";
    push @{ $self->{node_error} }, $input;
}

######################################################
sub node2_stdout
{
    my( $self, $input, $wid ) = @_;

    DEBUG and warn "NODE2: $input\n";
    if( $input =~ /PID=(\d+)/ ) {
        $self->{node2_PID} = $1;
        END { kill 15, $self->{node2_PID} if $self->{node2_PID} };
    }
    else {
        push @{ $self->{node2_output} }, $input;
    }
}

sub node2_stderr
{
    my( $self, $input, $wid ) = @_;

    DEBUG and warn "NODE2 err: $input\n";
    if( $input =~ /An address was in use/ ) {
        $self->{node2_in_use}++;   
    }
    else {
        push @{ $self->{node2_error} }, $input;
    }
}



######################################################
sub server_stdout
{
    my( $self, $input, $wid ) = @_;

    DEBUG and warn "SERVER: $input\n";

    if( $input =~ /^I am (\d+)/ ) {
        $self->{server_PID} = $1;
        END { kill 15, $self->{server_PID} if $self->{server_PID} };
    }
    elsif( $input =~ /^I am ([-.\w]+-[0-9a-f]+)/ ) {
        $self->{server_ID} = $1;
    }
    elsif( $input =~ /^Remote kernel (?:alias )?([-:.\w]+) went HELLO!/ ) {
        $self->{server_connect}{$1}++;
    }
    elsif( $input =~ m(^Connected poe://(Client\d+)/me/pulse) ) {
        $self->{client_connect}{$1}++;
    }
    elsif( $input =~ /^(Running server|Sending time|Done|Server exited).../ ) {
        $self->{$1}++;
    }
    elsif( $input =~ m(^poe://(Client\d+)/me/pulse -- ) ) {
        $self->{client_pulse}{$1}++;
    }
    elsif( $input =~ /\S/ ) {
        push @{ $self->{server_output} }, $input;
    }
}

sub server_stderr
{
    my( $self, $input, $wid ) = @_;

    if( $input =~ /Not connected to proxy-node/ ) {
        die "Race-condition in server! ($input)\n";
    }

    DEBUG and warn "SERVER err: $input\n";
    push @{ $self->{server_error} }, $input;
}

sub server_close
{
    my( $self, $wid ) = @_;
    $self->{server_close}++;
}

######################################################
sub client_stdout
{
    my( $self, $input, $wid ) = @_;

    DEBUG and warn "CLIENT: $input\n";

    if( $input =~ /^I am (\d+)/ ) {
        $self->{client_PID} = $1;
        END { kill 15, $self->{client_PID} if $self->{client_PID} };
    }
    elsif( $input =~ /^I am ([-.\w]+-[0-9a-f]+)/ ) {
        $self->{client_ID} = $1;
    }
    elsif( $input =~ /^\* connection to ([-:.\w]+)/ ) {
        $self->{client_connect}{$1}++;
    }
    elsif( $input =~ m(^\*\*\*\*\* Connected to (Pulse)) ) {
        $self->{client_connect_Pulse}++;
    }
    elsif( $input =~ m(^\*\*\*\*\* Disconnected from (Pulse)) ) {
        $self->{client_connect_Pulse}--;
    }
    elsif( $input =~ m(^HONK HONK) ) {
        $self->{client_honk}++;
    }
    elsif( $input =~ /^(Running client|Creating sessions).../ ) {
        $self->{$1}++;
    }
    elsif( $input =~ m(Subscribed to HASH.+ on ([-:.\w]+)) ) {
        $self->{client_subscribe}{$1}++;
    }
    elsif( $input =~ m(\|+ Foreign time is (.+)) ) {
        $self->{client_time} = $1;
    }
    elsif( $input =~ m([+|]+\* disconnection from Pulse) ) {
        $self->{client_disconnect} = $1;
    }
    elsif( $input =~ /\S/ ) {
        push @{ $self->{client_output} }, $input;
    }
}

sub client_stderr
{
    my( $self, $input, $wid ) = @_;

    DEBUG and warn "CLIENT err: $input\n";
    push @{ $self->{client_error} }, $input;
}

sub client_close
{
    my( $self, $wid ) = @_;
    $self->{client_close}++;

    DEBUG and warn "Client close: shutdown\n";

    delete $self->{client_wheel};
    delete $self->{server_wheel};
    delete $self->{node_wheel};

}



1;

__END__

$Log$


